@@ -1,8 +1,8 @@
+import json
+import re
 import urlparse
 import urllib
 from StringIO import StringIO
-import json
-import bson
 from wsgit.request import WebRequest
 
 
@@ -27,8 +27,8 @@ class Environ(object):
             'REMOTE_PORT', 'SERVER_NAME', 'SERVER_PORT', 'REQUEST_METHOD'
         ):
             environ[key] = getattr(self, '_get_%s' % key.lower())()
+        environ.update(self._headers_for_environ())
         environ.update(self._get_wsgi_io_dict())
-        environ.update(self.request.headers)
         self.environ = environ
         return environ
 
@@ -72,14 +72,25 @@ class Environ(object):
             'wsgi.version': (1, 0)
         }
 
+    def _headers_for_environ(self):
+        return dict(
+            ('HTTP_'+key.upper().replace('-', '_'), value)
+            for key, value in self.request.headers.items()
+        )
+
 
 class WSGIHandler(object):
 
-    def __init__(self):
-        self.result = dict()
+    def __init__(self, request_handler):
+        self.result = dict(response=dict())
+        self.headers = dict()
+        self.request_handler = request_handler
 
     def _start_response(self, status, response_headers):
         code, reason = status.split(' ')[0], ' '.join(status.split(' ')[1:])
+        self.headers.update(dict(
+            (key, value.strip()) for key, value in response_headers
+        ))
         self.result['status'] = dict(code=code, reason=reason)
 
     def call_application(self, app, environ):
@@ -90,7 +101,12 @@ class WSGIHandler(object):
         finally:
             if hasattr(app_iter, 'close'):
                 app_iter.close()
-        return bson.dumps(self.result)
+        self.result['headers'] = dict(
+            (key, value)
+            for key, value in self.headers.iteritems()
+            if key.lower() in self.request_handler.allow_headers
+        )
+        return self.result
 
     def _update_result(self, item):
         try:
@@ -99,4 +115,4 @@ class WSGIHandler(object):
             self.result['no_json_response'] = \
                 self.result.get('no_json_response', '') + item
         else:
-            self.result.update(response)
+            self.result['response'].update(response)
